#
# Copyright (c) 2011 Red Hat, Inc.
#
# This software is licensed to you under the GNU Lesser General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (LGPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of LGPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/lgpl-2.0.txt.
#
# Jeff Ortel <jortel@redhat.com>
#

"""
Contains AMQP message producer classes.
"""

from logging import getLogger

from qpid.messaging import Message

from gofer.messaging.adapter.model import BaseSender
from gofer.messaging.adapter.qpid.endpoint import Endpoint


log = getLogger(__name__)


class Sender(BaseSender):
    """
    An AMQP message sender.
    """

    def __init__(self, url=None):
        """
        :param url: The broker url.
        :type url: str
        """
        BaseSender.__init__(self, url)
        self._endpoint = Endpoint(url)
        self._link = None

    def endpoint(self):
        """
        Get a concrete object.
        :return: A concrete object.
        :rtype: BaseEndpoint
        """
        return self._link or self._endpoint

    def link(self, messenger):
        """
        Link to another messenger.
        :param messenger: A messenger to link with.
        :type messenger: gofer.messaging.adapter.model.Messenger
        """
        self._link = messenger.endpoint()

    def unlink(self):
        """
        Unlink with another messenger.
        """
        self._link = None

    def send(self, route, content, ttl=None):
        """
        Send a message.
        :param route: An AMQP route.
        :type route: str
        :param content: The message content
        :type content: buf
        :param ttl: Time to Live (seconds)
        :type ttl: float
        """
        message = Message(content=content, durable=True, ttl=ttl)
        channel = self.channel()
        sender = channel.sender(route)
        sender.send(message)
        sender.close()
        log.debug('sent (%s)', route)
